@@ -1,5 +1,6 @@
 #include "ccv.h"
 #include "ccv_internal.h"
+#include <sys/time.h>
 #ifdef HAVE_GSL
 #include <gsl/gsl_rng.h>
 #include <gsl/gsl_randist.h>
@@ -9,6 +10,17 @@
 #include <omp.h>
 #endif
 
+const ccv_bbf_param_t ccv_bbf_default_params = {
+	.interval = 5,
+	.min_neighbors = 2,
+	.accurate = 1,
+	.flags = 0,
+	.size = {
+		24,
+		24,
+	},
+};
+
 #define _ccv_width_padding(x) (((x) + 3) & -4)
 
 static inline int _ccv_run_bbf_feature(ccv_bbf_feature_t* feature, int* step, unsigned char** u8)
@@ -1202,9 +1214,9 @@ ccv_array_t* ccv_bbf_detect_objects(ccv_dense_matrix_t* a, ccv_bbf_classifier_ca
 			ccv_sample_down(pyr[i * 4 - next * 4], &pyr[i * 4 + 3], 0, 1, 1);
 		}
 	ccv_array_t* idx_seq;
-	ccv_array_t* seq = ccv_array_new(64, sizeof(ccv_comp_t));
-	ccv_array_t* seq2 = ccv_array_new(64, sizeof(ccv_comp_t));
-	ccv_array_t* result_seq = ccv_array_new(64, sizeof(ccv_comp_t));
+	ccv_array_t* seq = ccv_array_new(sizeof(ccv_comp_t), 64, 0);
+	ccv_array_t* seq2 = ccv_array_new(sizeof(ccv_comp_t), 64, 0);
+	ccv_array_t* result_seq = ccv_array_new(sizeof(ccv_comp_t), 64, 0);
 	/* detect in multi scale */
 	for (t = 0; t < count; t++)
 	{
@@ -1359,7 +1371,7 @@ ccv_array_t* ccv_bbf_detect_objects(ccv_dense_matrix_t* a, ccv_bbf_classifier_ca
 	/* the following code from OpenCV's haar feature implementation */
 	if (params.flags & CCV_BBF_NO_NESTED)
 	{
-		result_seq2 = ccv_array_new(64, sizeof(ccv_comp_t));
+		result_seq2 = ccv_array_new(sizeof(ccv_comp_t), 64, 0);
 		idx_seq = 0;
 		// group retrieved rectangles in order to filter out noise
 		int ncomp = ccv_array_group(result_seq, &idx_seq, _ccv_is_equal, 0);
